// ************************************************************
//					Potmot ALPS v1.2
// ************************************************************
//      Adaptation d'un projet trouvé sur le net 
//      pour µC ATMEGA8535 de chez ATMEL
//      Modif de:               Totof      Copyleft 2012
//      le post qui en parle et pour découvrir sa génèse est sur Audiyofan.org
//      http://www.audiyofan.org/forum/viewtopic.php?f=60&t=8502&start=0     
// *************************************************************
//                                      Projet initial
// *************************************************************      
// 	Autor:			Mario Richter
// 	Dateiname:	preamp.c
// 	Projekt:		HiFi Audio Preamplifier
// 	Datum:			Marz 2009
// 	Target-MCU:	ATmega8
//
// BESCHREIBUNG
// 	Software for den HiFi Audio Preamplifier. Die Software k�mmert
// 	sich um die Ansteuerung des Displays, wertet die Eingangssignale
//  vom IR-Sensor und von den Tastern aus und steuert die Relais der
//  Eingangsmatrix sowie den Treiber f�r das Motorpoti.
//
// PORTBELEGUNG
//
//  Relaismatrix:   PD1..PD6    Kanal1..7     (H = canal sélectionné)
//
//  Display:        PA4         D4            Données (4 Bit-Mode)
//                  PA5         D5
//                  PA6         D6
//                  PA7         D7
//                  PA3         E             Enable
//                  PA2         RW            Read/Write
//                  PA1         RS            Register Select
//
//  Taster:         PC1         Mute          sourdine
//                  PC2         Source        Sélection du canal
//
//  Motorsteuerung: PC3         MOT_ENABLE    Enable
//                  PC4         MOT_LEFT      Rotation gauche (diminution)
//                  PC5         MOT_RIGHT     Rotation droite (augmentation)
//
//  IR-Sensor:      PA0         IR            Sharp GP1U5
//
//  Mute-Schaltung: PD0         MUTE          H = sourdine
//
// ************************************************************

#include <avr/io.h>		      // I/O Ports
#include <avr/interrupt.h>            // Interrupts
#include <stdint.h>
#define F_CPU 4000000UL               // Clock frequency
#include <avr/eeprom.h>

#include "dogm.h"                     // Routinen for DOGM LCD
#include "rc5.h"                      // Routinen for RC5 IR Receiver

#define DDR(x) (*(&x - 1))      /* address of data direction register of port x */
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
    /* on ATmega64/128 PINF is on port 0x00 and not 0x60 */
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
  #define PIN(x) (*(&x - 2))    /* address of input register of port x          */
#endif

// Délai de retard sur action
#define TICK_10MS     (uint8_t)(F_CPU * 10e-3 / 512)
#define TICK_1MS      (uint8_t)(F_CPU * 1e-3 / 512)

#define MENU_DELAY    200     // 200 * 10ms = 2s

// Définition des ports
#define SOURCE_PORT   PORTD

#define MUTE_PORT     PORTD
#define MUTE          PD0

#define SWITCH_PORT   PORTC
#define SWITCH_MUTE   PC1
#define SWITCH_SRC    PC2

#define MOTOR_PORT    PORTC
#define MOTOR_ENA     PC3
#define MOTOR_LEFT    PC4
#define MOTOR_RIGHT   PC5

// Nombre de sources
#define MAX_SOURCE        2

// Délai anti-rebond
#define SWITCH_DEBOUNCE_CNTR    10    // 10 * 1ms = 10ms
#define SWITCH_LONG_PUSH        1000  // 1000 * 1ms = 1s

// Codes RC5
#define RC5_VOLUME_UP   16            // Volume +
#define RC5_VOLUME_DN   17            // Volume -
#define RC5_CHANNEL_UP  32            // Canal +
#define RC5_CHANNEL_DN  33            // Canal -
#define RC5_MUTE        13            // Sourdine

#define RC5_DELAY       20            // Durée des commandes RC5 = 20*10ms = 200ms hors tout
#define EEPROM_DELAY    1000          // 1000 * 10ms = 10s
#define CHANGE_DELAY    2             // 2*10ms = 20ms (2 fois pour minimum + cache)

uint8_t ticker10ms;                   // 10ms ticker
uint8_t ticker1ms;                    // 1ms ticker
uint16_t menu_delay;                  // Temps d'affichage du menu

uint8_t menu;                         // Affichage du menu ( 0= principal, 1= sélection de la source )
uint8_t source, source_new;           // Sélection de la source (0..5, 0=CD, 1=Tuner...)
uint8_t muting, muting_new;           // Sourdine (1) ou autre (0)

                      
uint8_t source_eep;                   // Variables de l'EEPROM
uint8_t mute_eep;

uint16_t source_eep_dly;
uint16_t mute_eep_dly;

uint8_t menu_dsp, source_dsp;         // Affichage des paramètres: affiche les paramètres actuel
uint8_t muting_dsp;                   // Seront affichées sur l'écran LCD
uint8_t vol_stat_dsp;                 // 1= Actualisation nécessaire

uint8_t mute_dbcnt;                   // Antirebond pour le bouton mute
uint16_t src_dbcnt;                   // Antirebond pour le bouton Source (+ Détection appui long)

uint8_t vol_up_delay;                 // 100ms durée du compteur pour volume +
uint8_t vol_dn_delay;                 // 100ms durée du compteur pour volume -

uint8_t source_delay;                 // Durée du compteur pour action "source"
uint8_t muting_delay;                 // Durée du compteur pour action "sourdine"

uint8_t rc5_delay;                    // délai du compteur pour les commandes RC5


// Routine principale
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

int main(void)
{
  unsigned int i;

  vol_up_delay = 0;
  vol_dn_delay = 0;
  rc5_delay = 0;

  source_eep_dly = 0;
  mute_eep_dly = 0;

  menu_dsp = 99;                      // force le rafraîchissement de l'affichage

  // Initialisation des ports
  DDR(SOURCE_PORT)  = 0x7F;           // PD6..PD0 choix de la direction des ports comme sorties (Relais)
  SOURCE_PORT       = 0x00;           // Tout les relais

  DDR(MOTOR_PORT)   = 0x38;           // PC3..PC5 choix de la direction des ports comme sorties (Moteur)
  MOTOR_PORT        = 0x00;           // Toutes les sorties

  DDR(MUTE_PORT)    = 0X01;           // PD0 choix de la direction du ports comme sortie (Sourdine)
  MUTE_PORT         = 0x01;           // D'abord en sourdine (!)        

  // TIMER0 pour préparer le récepteur RC5
  TCCR0 = 1<<CS02;                    // Division par 256
  TIMSK = 1<<TOIE0;                   // Activer les interruptions

  // Initialiser l'affichage du message et message de démarage
  dogm_init();
  dogm_puts("Potmot ALPS V1.3");
  dogm_gotoxy(0, 1);
  dogm_puts(" Audiyofan.org ");

  // attendre 1s
  _delay_ms(1000.0);
  dogm_clear();

  // test moteur
  dogm_puts("  Test moteur   ");
  dogm_gotoxy(0, 1);
  dogm_puts("  Aller-retour  ");

  _delay_ms(1000.0);
  dogm_clear();

  dogm_puts("     Duree      ");
  dogm_gotoxy(0, 1);
  dogm_puts("  100ms-100ms  ");

  _delay_ms(1000.0);
  dogm_clear();

  MOTOR_PORT &= ~(1<<MOTOR_RIGHT);
      MOTOR_PORT |= ((1<<MOTOR_ENA) | (1<<MOTOR_LEFT));
  _delay_ms(100.0);
  
  MOTOR_PORT &= ~(1<<MOTOR_LEFT);
      MOTOR_PORT |= ((1<<MOTOR_ENA) | (1<<MOTOR_RIGHT));
  _delay_ms(100.0);

  // fin test moteur

  // Derniers réglages de l'EEPROM 
  source_eep = eeprom_read_byte(0x00);
  if(source_eep >= MAX_SOURCE)
  {
    // Pas de données disponibles dans la mémoire EEPROM, nous prenons des valeurs standard
    source = 0;
    source_new = 0;

    SOURCE_PORT &= ~(0x7E);
    SOURCE_PORT |= 0x02;
  }
  else
  {
    source = source_eep;
    source_new = source;

    switch(source)            // Sélectionnez un canal en fonction du choix de la source
    {
    case 0:   SOURCE_PORT &= ~(0x3C);   SOURCE_PORT |= 0x02;  break;
    case 1:   SOURCE_PORT &= ~(0x3A);   SOURCE_PORT |= 0x04;  break;
    case 2:   SOURCE_PORT &= ~(0x36);   SOURCE_PORT |= 0x08;  break;
    case 3:   SOURCE_PORT &= ~(0x2E);   SOURCE_PORT |= 0x10;  break;
    case 4:   SOURCE_PORT &= ~(0x1E);   SOURCE_PORT |= 0x20;  break;
    default:  SOURCE_PORT &= ~(0x3E);
    }
  }  

  mute_eep = eeprom_read_byte(0x00);
  if(mute_eep == 1)
  {
    muting = 1;
    muting_new = 1;
  }
  else
  {
    muting = 0;
    muting_new = 0;
  }

  // Délai pour changer l'état sourdine
  _delay_ms(10.0);
  MUTE_PORT &= ~(1<<MUTE);

  sei();

  while(1)
  {
    cli();
    i = rc5.data & 0x3f;     // read two bytes from interrupt !
    rc5.data = 0;
    sei();


    // Décodage des commandes RC5
    // -----------------------------
    switch(i)
    {
    case RC5_VOLUME_UP:         // Volume +
      vol_up_delay = 15;        // Commande maintenue pendant 15*10ms = 150ms
      vol_dn_delay = 0;
      vol_stat_dsp = 1;         // Rafraîchissement de l'écran nécessaire
      break;

    case RC5_VOLUME_DN:         // Volume -
      vol_dn_delay = 15;        // Commande maintenue pendant 15*10ms = 150ms
      vol_up_delay = 0;
      vol_stat_dsp = 1;         // Rafraîchissement de l'écran nécessaire
      break;

    case RC5_CHANNEL_UP:            // Canal +
      if(!rc5_delay)                // ne fonctionne que si la dernière commande a été faite il y a plus de 200ms
      {
        rc5_delay = RC5_DELAY;      // Fixation du délai pour RC5
        menu = 1;                   // Affichage du menu sélection des sources à l'écran
        menu_delay = MENU_DELAY;    // Délai d'affichage du menu

        source_eep_dly = 0;
                
        if(source_new == MAX_SOURCE)  // Compteur de sélection des sources; interrupteur de débordement de retour à 0 si nombre de source maxi atteinte
          source_new = 0;
        else
          source_new += 1;
      }
      break;

    case RC5_CHANNEL_DN:            // CANAL -
      if(!rc5_delay)                // ne fonctionne que si la dernière commande a été faite il y a plus de 200ms
      {
        rc5_delay = RC5_DELAY;      // Délai de la commande RC5
        menu = 1;                   // Affichage du menu sélection des sources à l'écran
        menu_delay = MENU_DELAY;    // Délai d'affichage du menu
        source_eep_dly = 0;
        
        if(source_new == 0)         // Si le compteur de sélection des sources = 0 , dito remonte au maximum du nombre de source
          source_new = MAX_SOURCE;
        else
          source_new -= 1;
      }
      break;

    case RC5_MUTE:                  // Sourdine
      if(!rc5_delay)                // ne fonctionne que si la dernière commande a été faite il y a plus de 200ms
      {
        rc5_delay = RC5_DELAY;      // Fixé au délai RC5 
        muting_new ^= 0x01;         // Basculer en sourdine
      }
    }

    // Contrôle des moteurs
    // -----------------------------
    if(vol_dn_delay)            // vol_dn_delay -> le moteur tourne à gauche
    {
      MOTOR_PORT &= ~(1<<MOTOR_RIGHT);
      MOTOR_PORT |= ((1<<MOTOR_ENA) | (1<<MOTOR_LEFT));
    }
    else if(vol_up_delay)       // vol_up_delay -> le moteur tourne à droite
    {
      MOTOR_PORT &= ~(1<<MOTOR_LEFT);
      MOTOR_PORT |= ((1<<MOTOR_ENA) | (1<<MOTOR_RIGHT));
    }
    else                        // Le moteur est arrêté
      MOTOR_PORT &= ~((1<<MOTOR_ENA) | (1<<MOTOR_LEFT) | (1<<MOTOR_RIGHT));

    // Sélection de la source
    // -----------------------------
    if(muting)
      SOURCE_PORT &= ~(0x7E);   // Si la sourdine est activé ==> tous les relais sont décollés
    else
    {
      switch(source)            // autrement, activation du relais selon la source sélectionnée
      {
      case 0:   SOURCE_PORT &= ~(0x3C);   SOURCE_PORT |= 0x02;  break;
      case 1:   SOURCE_PORT &= ~(0x3A);   SOURCE_PORT |= 0x04;  break;
      case 2:   SOURCE_PORT &= ~(0x36);   SOURCE_PORT |= 0x08;  break;
      case 3:   SOURCE_PORT &= ~(0x2E);   SOURCE_PORT |= 0x10;  break;
      case 4:   SOURCE_PORT &= ~(0x1E);   SOURCE_PORT |= 0x20;  break;
      default:  SOURCE_PORT &= ~(0x3E);
      }

    }

    // 1ms-Timer
    // -----------------------------
    if(ticker1ms >= TICK_1MS)
    {
      ticker1ms = 0;

      // Antirebond du bouton sourdine
      if(!(PIN(SWITCH_PORT) & (1<<SWITCH_MUTE)))
      {
        if(mute_dbcnt <= SWITCH_DEBOUNCE_CNTR)
          mute_dbcnt++;

        if(mute_dbcnt == SWITCH_DEBOUNCE_CNTR)
          muting_new ^= 0x01;
      }
      else
        mute_dbcnt = 0;

      // antirebond du bouton source
      if(!(PIN(SWITCH_PORT) & (1<<SWITCH_SRC)))
      {
        if(src_dbcnt <= SWITCH_LONG_PUSH)
          src_dbcnt++;

      }
      else
      {
        // En cas d'appui court sur le bouton source
        if((src_dbcnt > SWITCH_DEBOUNCE_CNTR) && (src_dbcnt < SWITCH_LONG_PUSH))
        {
          menu = 1;                     // Affichage du menu sélection des sources à l'écran
          menu_delay = MENU_DELAY;      // Délai d'affichage du menu

          source_eep_dly = 0;
          
          if(source_new == MAX_SOURCE)  // Compteur de sélection des sources; interrupteur de débordement de retour à 0 si nombre de source maxi atteinte
            source_new = 0;
          else
            source_new += 1;
        }

        src_dbcnt = 0;                  // Remise à zéro de l'antirebond du bouton source
      }
    }

    // 10ms-Timer
    // -----------------------------
    if(ticker10ms >= TICK_10MS)
    {
      ticker10ms = 0;

      // Avant de modifier la source activation de la sourdine
      if((source != source_new) && (source_delay == 0))
      {
        source_delay = CHANGE_DELAY;
        MUTE_PORT |= (1<<MUTE);
      }

      // changement de source
      if(source_delay)
      {
        source_delay--;

        if(source_delay == 0)
        {
          // ÉTAPE 1: changer de source et la remplacer par la source sélectionnée
          if(source != source_new)
          {
            source = source_new;
            source_delay = CHANGE_DELAY;
          }
          // ÉTAPE 2: désactivé la sourdine
          else
          {
            MUTE_PORT &= ~(1<<MUTE);
          }
        }
      }      

      // Commutateur de sourdine
      if((muting != muting_new) && (muting_delay == 0))
      {
        muting_delay = CHANGE_DELAY;
        MUTE_PORT |= (1<<MUTE);

        // Lorsque la sourdine est désactivé , le relais est aussi désactivé
        // (la sourdien est de toute façon déjà active)
        if(muting_new == 0)
          muting = 0;
      }

      // Sourdine
      if(muting_delay)
      {
        if(--muting_delay == 0)
        {
          muting = muting_new;

          // Si la sourdien est activé , activation du relais de sourdine 
          if(muting == 0)
            MUTE_PORT &= ~(1<<MUTE);
        }
      }

      // Temporisation du volume + délai d'entraînement du moteur
      if(vol_up_delay > 0)
      {
        vol_up_delay--;

        if(vol_up_delay == 0)           // Si arrive à zéro mise à jour de l'affichage
          vol_stat_dsp = 1;
      }

      // Temporisation du volume - délai d'entraînement du moteur
      if(vol_dn_delay > 0)
      {
        vol_dn_delay--;

        if(vol_dn_delay == 0)           // Si arrive à zéro mise à jour de l'affichage
          vol_stat_dsp = 1;
      }

      // Le sous-menu apparaît
      if(menu_delay)
      {
        menu_delay--;

        if(menu_delay == 0)             // Quand le délai est écoulé, revenir au menu principal
        {
          //source = source_new;
          menu = 0;
        }
      }

      // Délai des commandes RC5
      if(rc5_delay)
        rc5_delay--;

      // Ecriture de l'EEPROM
      if(source != source_eep)          // Choix d'une nouvelle source ?
      {
        if(++source_eep_dly == EEPROM_DELAY)    // Ajouter ensuite le délai dans l'EEPROM
        {
          eeprom_write_byte(0x00, source);
          source_eep = source;
          source_eep_dly = 0;
        }
      }

    }

    // Tâche d'affichage : menu principal
    // -----------------------------
    if(menu == 0)
    {
      if(menu != menu_dsp)      // Affichage vierge, le menu principal, tout à redessiner
      {
        dogm_clear();

        switch(source)          // Représente les choix de sources sélectionnées
        {
        case 0:   dogm_puts("      IVTCY     ");  break;
        case 1:   dogm_puts("   CD direct    ");  break;
        case 2:   dogm_puts("       PC       ");  break;
        case 3:   dogm_puts("      Aux1      ");  break;
        case 4:   dogm_puts("      Aux2      ");  break;
        }

        dogm_gotoxy(0, 1);

        if(vol_dn_delay)        // Les changements de volume s'affiche sur l'écran LCD
          dogm_puts("VOL-  ");
        else if(vol_up_delay)
          dogm_puts("VOL+  ");
        else
          dogm_puts("      ");

        if(muting)              // Affichage du statut sourdine
          dogm_puts("MUTE");

        menu_dsp = menu;        // Rappelez-vous, ce qui est montré !
        source_dsp = source;
        muting_dsp = muting;
      }
      else                        // Le menu principal est déjà affichée, examiner ce qui doit être mis à jour
      {
        if(source_dsp != source)  // Vérifiez si la source correspond, si non, alors ré-afficher
        {
          source_dsp = source;    // Mettre à jour l'affichage de la source
          dogm_gotoxy(0, 0);

          switch(source)          // Les choix sélectionnées représentent
          {
          case 0:   dogm_puts("      IVTCY     ");  break;
          case 1:   dogm_puts("   CD direct    ");  break;
          case 2:   dogm_puts("       PC       ");  break;
          case 3:   dogm_puts("      Aux1      ");  break;
          case 4:   dogm_puts("      Aux2      ");  break;
          }
        }

        if(vol_stat_dsp)        // Etat qui indique si il faut afficherVol + / Vol- ou écran vide si rien
        {
          vol_stat_dsp = 0;
          dogm_gotoxy(6, 1);

          if(vol_dn_delay)      // Affichage des commandes de volume sur le LCD
            dogm_puts("VOL-  ");
          else if(vol_up_delay)
            dogm_puts("VOL+  ");
          else
            dogm_puts("      ");
        }

        if(muting_dsp != muting)  // Vérifie l'état sourdine, affichage ou rien
        {
          muting_dsp = muting;
          dogm_gotoxy(12, 1);

          if(muting)              // affiche "mu
            dogm_puts("MUTE");
          else
            dogm_puts("    ");
        }
      }
    }

    // Routine d'affichage, menu selection
    // -----------------------------
    if(menu == 1)
    {
      if(menu_dsp != menu)        // La sélection de la source n'est pas visible (affich
      {
        source_dsp = source_new;  // Affiche une phrase pour sélectionner une nouvelle source
        menu_dsp = menu;

        dogm_clear();
        dogm_puts("Selection Source:");

        dogm_gotoxy(0, 1);
        switch(source_new)        // Les choix sélectionnées représentent
        {
          case 0:   dogm_puts("      IVTCY     ");  break;
          case 1:   dogm_puts("   CD direct    ");  break;
          case 2:   dogm_puts("       PC       ");  break;
          case 3:   dogm_puts("      Aux1      ");  break;
          case 4:   dogm_puts("      Aux2      ");  break;
          }
      }
      else                        // Une sélection de la source est déjà affichée
      {
        if(source_dsp != source_new)
        {
          source_dsp = source_new;
          dogm_gotoxy(0, 1);

          switch(source_new)      // Afficher la source qui doit être sélectionnée
          {
          case 0:   dogm_puts("      IVTCY     ");  break;
          case 1:   dogm_puts("   CD direct    ");  break;
          case 2:   dogm_puts("       PC       ");  break;
          case 3:   dogm_puts("      Aux1      ");  break;
          case 4:   dogm_puts("      Aux2      ");  break;
          }
        }
      }
    }
  }
}

// Timer0 ISR
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ISR(SIG_OVERFLOW0)
{
  ticker1ms++;
  ticker10ms++;

  rc5_scheduler();
}

